import org.mcavallo.opencloud.formatters.*;
import org.mcavallo.opencloud.*;
import org.mcavallo.opencloud.filters.*;
import org.mcavallo.opencloud.util.*;

import java.util.Date;

import org.mcavallo.opencloud.*;
import processing.serial.*;
import processing.video.*;

int w = 800;
int h = 600;
Serial port;
Cloud cloud;
float  maxWordDisplaySize = (float)40.0;
int BUFFSIZE = 32;
char[] buffer = new char[BUFFSIZE];
int wordStart = 0;
int index = 0;
boolean pause = false;
boolean recording = false;
MovieMaker mm;
int movieIndex = 1;
int imageIndex = 1;

void setup() {
  size(w, h);
  smooth();
  noStroke();
  frameRate(24);

  String portName = Serial.list()[0];
  //println(portName);
  port = new Serial(this, portName, 57600);

  cloud = new Cloud();
  cloud.setMaxWeight(64.0); // max font size
  cloud.setMaxTagsToDisplay(400);
  cloud.setTagCase(Cloud.Case.UPPER);
  String[] commonWords = {
    "THE", "AND", "FOR", "OUR", "YOU", "ARE", "BUT", "WHAT", "WHERE", "FROM", "THIS", "WAS", "WITH", "THAT", 
    "THAN", "NOT", "IT'S", "YOUR", "ANNOUNCER", "APPLAUSE"
  };
  cloud.addInputFilter(new MinLengthFilter(3));
  cloud.addInputFilter(new DictionaryFilter(commonWords));
}

void draw() {
  String word;
  char c;
  word = null;
  if (port.available() > 0) {
    c = port.readChar();
    if (c == ' ') {
      if (index > 0) {
        word = new String(buffer, 0, index);
        index = 0;
      }
    } 
    else {
      buffer[index++] = c;
    }
    if (word != null) {
      word = word.trim().toUpperCase();
      println(word);
      if (!pause) {
        cloud.addTag(word);
      }
    }
  }
  drawCloud();
  if (recording) {
    mm.addFrame();
  }
}

void mousePressed() {
  cloud.clear();
  background(255);
}

void keyPressed() {
  if (key == ' ') {
    pause = !pause;
    if (pause) {
      println(">>>>>>>>>> paused");
    } 
    else {
      println(">>>>>>>>>> unpaused");
    }
  }
  if (key == 'r') {
    recording = !recording;
    if (recording) {
      String movieName = "TVCloud" + movieIndex++ + ".mov";
      println(">>>>>>>>>> recording " + movieName);
      mm = new MovieMaker(this, width, height, "../" + movieName, 30, MovieMaker.JPEG, MovieMaker.BEST, 15);
    } 
    else {
      println(">>>>>>>>>> recording stopped");
      mm.finish();
    }
  }
  if (key == 's') {
    String imageName = "TVCloud" + imageIndex++ + ".png";
    save("../" + imageName);
  }
}

void drawCloud() {
  float xMargin = 40;
  float ySpacing = 50;
  float xPos = xMargin; // initial x position
  float yPos = ySpacing;      // initial y position
  Date now = new Date();
  long age;

  background(255);
  for (Tag tag : cloud.tags()) {
    String tName = tag.getName();
    float tWeight = (float) tag.getWeight();
    float wordSize =  maxWordDisplaySize * (pow((float) (tWeight/maxWordDisplaySize), (float)0.5));
    //float wordSize =  maxWordDisplaySize * (tWeight/maxWordDisplaySize);

    // Draw the word
    textSize(wordSize);
    if ((xPos + textWidth(tName)) > width) {
      xPos  = xMargin;
      yPos += ySpacing;
    }
    age = now.getTime() - tag.getDate().getTime();
    if (age < 10) {
      fill(0, 255, 255);
      rect(xPos, yPos-textAscent(), textWidth(tName), textAscent());
      fill(255);
    } 
    else {  
      fill(0, map(age, 10, 3000, 255, 0), 255);
    }
    text(tName, xPos, yPos);
    xPos += textWidth(tName) + 12.0;
  }
}

